
#include <stdio.h>
#include <stdarg.h>
#include "data_pairs.h"

class llm_node{
   public:
   float *pfV;        // input/output weight vector
   float *pfA;        // input/output matrix
   float fSqrDist;    // squared distance to input 
   float fSig;        // mean squared distance to input 
   float fActivation; // node "activation" (in range [0,1]) 
   float fEpsInp;     // node local input eps 
   float fEpsOut;     // node local output eps 
   float fEpsMat;     // node local matrix eps 
   llm_node(int iInp, int iOut) {
      pfV  = CALLOC(iInp+iOut,float);
      pfA  = CALLOC(iInp*iOut,float);
      fSig = 0;
      fEpsInp = fEpsOut = fEpsMat = 1;
   }
   ~llm_node(void) {
      free(pfA);  free(pfV);
   }
   int   iNumHits;    // absolute number of hits of node	       	     
   int   iIsAct;      // boolean, 1=active node, 0=inactive		    
};

class llm{
friend class llm_node;
protected:
   int iNodes;
   int iDimInp;
   int iDimOut;
   data_pairs *pTrainingData;
   data_pairs *pTestData;
private:
   int iBestNode, iBestNodeOnly;
   float fEpsAuto;
   /* global learning rates: */
   float fGlobalEpsInp, fGlobalEpsOut, fGlobalEpsMat;
   /* used = product of global & local */
   float fSigDecay;  // decay rate for fSig estimates */ 
   float fBeta;      /*  determines mode:
                      fBeta = 0  winner takes all
                      fBeta > 0  exponential weighting:
                                 G_i = f_i/sum_j f_j  with
                                 f_i = exp(-beta*squaredist_i)
                      fBeta < 0  polynomial weighting:
                                 f_i = (squaredist)^beta
                     */
   float *pfG;       // weighting coefficients G_i 
   float fGMin;      // nodes with smaller G values are ignored 
   float *pfNetOut;  // internal storage of last net output
   float *pfDV;
   float *pfMetric;
   float fG_A;       // weighting coefficient for matrix part in output update
   llm_node **ppNode;
   int legal_reference(int);
public:
   llm(int,int,int);
   virtual ~llm(void);
   virtual int eprintf(FILE*,const char*,...);
   void exec(float*, float*);
   void adapt_inputs(float*);
   void adapt_outputs(float*);
   void adapt_matrices(float*, float*);
   void   set_node_inp(int,float*);
   float *get_node_inp(int);
   float *get_aux(int);
   void   set_node_out(int,float*);
   float *get_node_out(int);
   void   set_node_matrix(int,float*);
   float *get_node_matrix(int);
   float *get_weights(void);
   int  get_num_nodes(void);
   void node_init(float);
   void node_norm(void);
   void node_AE(float, float, float);
   void node_expl(float, float, float, float);
   void set_eps_inp(float); 
   void set_eps_out(float); 
   void set_eps_mat(float); 
   void set_eps_auto(float); 
   void set_sig_decay(float); 
   void set_beta(float);
   void set_gA(float);
   void init_nodes(float,float,float,float,float,float);
   void init_sig(float);
   /* methods accessing dataset instances: */
   void set_training_data(data_pairs*);
   void set_test_data(data_pairs*);
   void use_training_data(void);
   void use_test_data(float*);
   int check_data_set(data_pairs*);
};
