typedef struct local_pca_{
   int iDim;        /* dimension of data space */
   int iNumNodes;   /* max nr of PCA nodes */
   float fSigma2;   /* sigma parameter: controls approximation */
   float fFac;      /* weight factor for euclicean dist to center:
                       dist^2 = fFac*dist_to_center^2 + 
                              (1-fFac)*dist_to_plane^2
                    */
   float fAvgDist;  /* avg data distance */
   int  iNumData;      /* nr of data vectors */
   float *pfData;   /* ptr to iNumData*iDim data elements. PUBLIC! */
   int   *piNode;   /* ptr to iNumData node assignment indices, one
                       for each data point */
   float *pfW;      /* ptr to iNumNodes*iDim elements of iNodes node
                       center vectors, each of dim iDim */
   float *pfMat;    /* ptr to iDim*iDim matrix elements to hold
                       autocorrelation matrix of current node
                    */
   int   *piDim;    /* ptr to iNumNodes subspace dimensionalities */
   float *pfBase;   /* ptr to iNumNodes*iDim*iDim elements of
                       iNumNodes*iDim eigenvectors, iDim per node
                    */
   float *pfL;      /* ptr to iNumNodes*iDim eigenvalues, iDim per node */
   float *pfAct;    /* ptr to iNumNodes scalar activity values */
   float *pfAssign; /* ptr to iNumData*iNumNodes assignment values */
   float *pfWrk;    /* workspace array of 2*iDim els */
} local_pca_t;

local_pca_t *lpca_create(int,int);
void  lpca_free(local_pca_t*);
void  lpca_init(local_pca_t*);
void  lpca_set_data(int,float*,local_pca_t*);
float lpca_update(local_pca_t*);

