/*****************************************************************************
 *
 * nst6.h  -  NST kernel layer 6: Variables
 *
 * Copyright Helge Ritter, 1992
 *
 * $Source: /vol/ni/src/cvsroot/nst5/nstsrc/nst6.h,v $
 *
 * $Log: nst6.h,v $
 * Revision 1.4  2002/01/15 09:28:52  helge
 * new macro vrotate(u,type,var): analogous to vput(), but additionally
 * shifts oldest stored variable out ("ring buffer mode").
 *
 * Revision 1.3  1997/04/02 14:07:31  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.2  1997/03/13  10:48:43  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST6_H
#define __NST6_H
#include "nst_types.h"

/* macros to put/get variables into/from local variable queue of
   a unit. "typ" must denote the type of the variable put/get'ted.
*/

#define vinit(u) (u)->qp = (vqueue_el *) (u)->var
/* must be called once before vget may be used. The first call
   of vget after vinit returns the variable that
   was inserted most recently into variable queue of unit u. */

#define vget(u,typ)  * ((typ *) vp_get(u))    
/* each call of vget returns the next variable in 
   the variable queue of unit u. After a call to vinit,
   the first variable returned is the last variable that
   was putt'ed into the queue. Calls of vget do not change the 
   contents of the queue itself, but move the queue pointer
   qp to the next item. */

#define vput(u,typ,x) v_put(u,sizeof(typ),&(x))
/* inserts variable x of type typ into variable queue of unit u.
   Location of insertion is behind the last variable fetched by
   vget (i.e., when vget has not been used since the last vinit,
   each new variable is inserted at the beginning of the
   variable queue).
   x must be an lvalue. Variables putt'ed can be
   retrieved in opposite order by macro vget. 
*/

#define vrotate(u,typ,x) v_rotate(u,sizeof(typ),&(x))

/* rotates variable queue: oldest element is re-used for
   accomodation of new one, u->nvar stays constant.
   This call is only permitted when u->nvar > 2 and
   this is not checked!
*/

#define vsize1(u,eltype,NUMEL) (fprintf(stderr,"isnotunit=%d size=%d\n",is_not_unit(u),sizeof(eltype)), (vqueue_el *) u->qp)->numel = -(NUMEL) 


#define vsize(u,eltype,NUMEL) ((vqueue_el *) u->qp)->numel = -(NUMEL) 
/**** CHANGED: earlier size field now replaced by direct specification
               of nr of allocated array elements (was necessary to
               be machine independent). The only users of
               the earlier size field were the fvput/fvget routines,
               and there the corresponding correction has now been made.
*/
/* @@@@ heavy bug removed!! Was u->var .. formerly and
   spoiled storage and load of variables */
 
/* A call of vsize provides a possibility to mark the most recently
   inserted variable as a pointer reference to an object of
   numel elements of type eltype. This information can then later 
   be used by copy and save routines to properly copy these memory
   locations that are extern al to the queue. A value numel=0 is used
   to suppress any such copying, e.g. for pointers to functions.
*/

/*  Orig def in nst3.h, here just for easier ref:

typedef struct def_vqueue_el{  
  void *el;
  int numel;    / * CHANGED: was size before * / 
  struct def_vqueue_el *prev, *next; 
     / * linear list in forward order,
       * circuilar list in backward order, thus 
       * u->var->prev points to last element of forward list.  
       * /
} vqueue_el;

*/

extern  void v_put(unitptr, size_t, void*);
extern  void v_rotate(unitptr, size_t, void*);
extern  void *vp_get(unitptr);
extern  void erasevars(unitptr);

#endif
