/****************************************************************************

   File nst_aux0.h:  auxiliary functions for drawing coordinate systems
                     and markers

   Copyright Helge Ritter 1995

*****************************************************************************/

#ifndef __NST_AUX0_H  /* header control */
#define __NST_AUX0_H

#include <stdio.h>
#include <math.h>

#define Notnull (x) (!x ? 1 : x)
#define Min(x,y) ((x > y) ? y : x))
#define Max(x,y) ((x > y) ? x : y))
#define rand 0

typedef struct nst_scaling_info_ {
	    int   iLeftMargin,iRightMargin,iBottomMargin,iTopMargin;
        short iSet;
        short iXMode,iYMode;  /* currently: >0 is linear, <0 is log scaling */
        float fScx,fScy,fInvScx,fInvScy,fX0,fY0;
        float fXAnchor,fYAnchor;  /* screen location in pixels through which
                                     drawn axes will pass */
} nst_scaling_info_t; 

extern int dat_to_pix(float,float,int*,int*,nst_scaling_info_t*);
extern int pix_to_dat(int,int,float*,float*,nst_scaling_info_t*);
/* NOTE: interface of nst_xyframe() still subject to change!! */
extern nst_scaling_info_t *nst_xyframe(int,float,float,float,float, 
  int,int,int,int,int,int,int,int,int*,nst_scaling_info_t*);
/* *************************************************************************
   draw a coordinate system from xmin to xmax and from ymin to ymax
   with xts marks on the x axis and yts marks on the y axis.
   The system is drawn in the window w with the x dimension xdim and
   the y dimension ydim.
************************************************************************* */
extern void draw_line(float,float,float,float,int,nst_scaling_info_t*);
extern void nst_add_label(float,float,char,char*,int,nst_scaling_info_t*);
extern void nst_mark(float,float,float,int,int,nst_scaling_info_t*);
/* *************************************************************************
   draw a quadratic marker symbol, centered at location (x,y)
   and of size siz. siz is specified in user units for the horizontal
   direction. The vertical size is adjusted such that each symbol
   appears quadratic.

   isym=0  point (size given by linewidth)
	1  square
	2  diamond
	3  triangle
	4  cross mark
	5  plus mark
	6  star burst
        7  circle

   w = window id
************************************************************************* */

/**************************************************************************

   3d drawing stuff (for both X11 and GL/OpenGL graphics):

**************************************************************************/

extern void nst_set_draw3d_methods(void(*)(float,float,float), 
                                 void(*)(float,float,float),
                                 void(*)(float,float,float),
                                 void(*)(float,float,float,float,float,float));
extern void nst_find_ticks(float,float,int,float*,float*,int*,char*); 
extern void nst_draw3d_ticks(float,float,float,int,int,float,char*,float); 
extern void nst_draw3d_frame(void(*)(float,float,float,float,float,float),unitptr);

typedef void nst_draw3d_frame_func_t (float,float,float,float,float,float);

#endif
