/***************************************************************************

     Version 2.0 of source file nst_aux2.h
     Text manipulation routines.

     Copyright by Helge Ritter 1992

***************************************************************************/
#ifndef __NST_AUX2_H
#define __NST_AUX2_H

#define MAXWIDTH  256   /* max TEXTWIDTH per line; DON'T increase!! */
#define SEPARATOR_CH '\n'

#define TEMP_FILE_TOKEN "TEMPFILE="
#define INVOKE_EDITOR  10   /* CTRL-J */
#define NST_ALL_LINES  -1   /* request refresh of all lines */
#define NST_NEXT_LINE  -2   /* request refresh of current and next line */
#define NST_PREV_LINE  -3   /* request refresh of current and previous line */
#define NST_ALL_LINES_SHADED  -4   /* redraw all lines shaded */
#define NST_ALL_LINES_AND_CURSOR  -5  

typedef struct linetype {
   char text[MAXWIDTH+1];    
   int linenum,linelen;
   struct linetype *next, *prev;
} linetype;

typedef struct _textwinstate {
   int xcw,ycw;  /* cursor pos in window rectangle,    (0,0)=upper left */
   int xct,yct;  /* cursor pos in full text rectangle, (0,0) =upper left */
   int dxw,dyw;  /* width and height of window rectangle */
   int dxt,dyt;  /* width and height of text rectangle */
   int dyt0;     /* if 0 : text rectangle is of fixed height
                    if >0: text rectangle is of variable height
                           and this is the minimal height,
                           i.e., dyt >= dyt0.             */
   int lastline;
   int ismarked, xmt, ymt; /* mark status and mark position in text */
   linetype *line;         /* pointer to current line yct */
   int refresh,mode; /* auxiliary information. 
       Currently recognized values for mode are: 
       NST_W_NOEDIT   accept only cursor commands 
       0    default value
   */
   int syntaxmode;  /* for syntax coloring */
} textwinstate;

extern  int is_noedit(char);
extern void nst_bell(void);
extern char *FilteredStrCpyOf(char*,char*);
extern  char *FormatFloatToStr(char*,float,int);
extern  char *FormatIntToStr(char*,int,int);
extern  int set_mark_at_cursor(textwinstate*);
extern  void nst_set_syntaxmode(int,textwinstate*);
extern  void set_mark_at_start(textwinstate*);
extern int set_cursorpos(textwinstate*,int,int);
extern void swap_cursor_with_mark(textwinstate*);
extern  void clear_mark(textwinstate*);
extern  int length_of_region(textwinstate*);
extern  void kill_region(textwinstate*);
extern  char *nst_copy_region_to_str(textwinstate*,char*);
extern  int yank_region(textwinstate*);
extern int set_region_limits(textwinstate*, int, int*, int*);
extern void find_sb_limits(textwinstate*,float*,float*);
extern int sb_position_text(textwinstate*,float);
extern  textwinstate* alloc_textwinstate(int wdx, int wdy, int tdx, int tdy);
extern  void free_textwinstate(textwinstate*);
extern  int mktext(char *oldtext, int maxlen, textwinstate *t);
extern  void mktextwinstate(char *text, textwinstate *t); 
extern  int mvcursor_x(int dx, textwinstate *t);
extern  int mvcursor_y (int dy, textwinstate *t);
extern  void move_text_window(char s, textwinstate *t);
extern  int insert_cr(textwinstate *t);
extern  int insert_ch(char s, textwinstate *t); 
extern  void insert_char(char s, textwinstate *t); 
extern  char cursor_bs(textwinstate *t);
extern  void nst_cursor_1d_to_2d(int,int*,int*,int,textwinstate*);
extern  int  nst_cursor_2d_to_1d(int,int,int,textwinstate*);
extern  void nst_move_text_down(int, textwinstate*);
extern  void nst_move_text_up(int, textwinstate*);
extern int is_marked(textwinstate *t);
extern textwinstate *copy_of_textwinstate(textwinstate *t);
extern int fstrstr(FILE*,char*);
extern char *strfile(FILE*,int);
extern void nst_set_query_function(void (*pQueryFunction)(char*));
extern void nst_set_goto_function(char *(*pGotoFunction)(int*));

#endif
