
/***********************************************************************

  include file nst_aux5.h: 2D-graphics

  Copyright Helge Ritter, 1996

***********************************************************************/

#ifndef __NST_AUX5_H
#define __NST_AUX5_H

/* define some values for the mode parameter of the nst_locate routine: */

#define NST_GETKEY   1
#define NST_MOUSE1   0  /* specifies which mouse button to use for
                          NST-Button1 (NST uses two buttons). Values
                          0,1,2 select left,middle,right mouse button */
#define NST_MOUSE2   2  /* selects right mouse button */
#define NST_MOUSE3   1  /* so far, NST tries to avoid usage of a third button */
#define NST_MOUSEX  16  /* fixed */
#define NST_MOUSEY  32  /* fixed */

/* the standard mode listens to the keyboard, the main button NST_MOUSE2,
   the locator xy values and nothing else. For the mask, the mouse button
   numbers 0,1,2 [left,middle,right] are translated into powers 2,4,8 */

#define NST_STDINP  (NST_GETKEY | (2<<NST_MOUSE1)| NST_MOUSEX | NST_MOUSEY)
#define NST_ALLINP  (NST_STDINP|(2<<NST_MOUSE2)|(2<<NST_MOUSE3))

/* NOTE: nowait mode is set by choosing the negative value */

extern int giNstScreenDx;
extern int giNstScreenDy;
extern float gfNstCurrentFontDx;
extern float gfNstCurrentFontDy;
extern float gfNstCurrentOffset;
extern float gfNstCurrentLineDy;
extern int giNstCurrentFont;

extern  int COLMAP;

#define EMPHASIZED_COLOR   (-31)  /* color for emphasized text [blue3] */
#define NORMAL_COLOR_CH      20   /* non-printable char codes to switch */
#define EMPHASIZED_COLOR_CH  21   /* colors */
 
extern char *gpcNstTitlePrefix;

/**** for compatibility (avoid to use): ***/
#define SCREEN_WIDTH  giNstScreenDx
#define SCREEN_HEIGHT giNstScreenDy
/******************************************/

extern  int iDISPLAY_ADAPTER;

#define RS6000_GT4X    1
#define RS6000_GTO     2
#define RS6000_3DPLUS  3
#define RS6000_3D      4
#define RS6000_NOGL    5
#define SGI_INDIGO     6
#define SGI_IRIS       7

typedef struct nst_winfo_{ /* local var of draw units: */

   /* 2D scaling parameters: */

   float fXScale,  fYScale;
   float fXOffset, fYOffset;
   int   iXMode,   iYMode;

   /* used in the following way to transform from data to pixel
      locations:   

      TX(s) =  iXMode>=0 ? s : log10(s);
      TY(s) =  iYMode>=0 ? s : log10(s);

      iX = TX(fX)*fXScale + fXOffset;
      iY = TY(fY)*fYScale + fYOffset;

   */

   float        fLastX,fLastY; /* remembers most recent pt drawn */
   unsigned int mode;
   unitptr      u, uWindowHost;
   int          isMainClient;
                /* flags main (=scaling) client of subwindow */
   int iWdw;    /* window identifier */
   int isOpen;  /* flag */
   int isSubwindow; /* flag */
   int iForeGround, iBackGround;  /* window fore/backgrounds 
                   (only used for main clients) */
   int iWhich;  /* for window hosts with multiple windows */
   int iType;   /* one of 0 (mono) 1 (RGB) 2 (RGB3d) */
   char *pcWdw; /* to identify window by string */
   char *pcFont; /* name of window font */
   /* window data for reopen: */
   int iXll, iYll, iDx, iDy; 
   int iOldWin; /* window valid before call to this unit */
   struct nst_winfo_  *pUnused;
   long int iBufState;  /* status of buffers for drawing */
   /* general drawing attributes: */
   int iColor,iSym,iSize,iHotSpot; /* iNum,iNumPts; */
   int *piPosInfo;
} nst_winfo_t;


extern nst_winfo_t *nst_get_winfo(void);
extern void nst_set_winfo(nst_winfo_t*);
extern FILE *nst_posinfo_file(void);
extern void nst_set_posinfo_file(FILE *fp);

extern void nst_hide_windows (int bHide);
extern int  nst_hidden_windows ();
extern nst_winfo_t *nst_init_window(char*,int,int,int,int,int,char*,int,int,int,unitptr);
extern nst_winfo_t *nst_init_rgb_window(char*,int,int,int,int,int,char*,int,int,int,unitptr);
extern nst_winfo_t *nst_init_rgb3d_window(char*,int,int,int,int,int,char*,int,int,int,unitptr);
extern int ctrl_window(int, nst_winfo_t*, unitptr);

extern nst_winfo_t *nst_find_hostwindow(char*, unitptr);
extern int nst_find_embeddor_window_of(int);
extern int nst_has_embedded_guis(unitptr);
extern char *nst_find_name_of_window(int);
extern int nst_find_window_from_title_and_host(char *pcTitle, unitptr u, unitptr uHost);

extern unitptr nst_begin_draw(nst_winfo_t*);
extern unitptr nst_begin_draw2(nst_winfo_t*,unitptr);
extern unitptr nst_begin_cmap_draw(nst_winfo_t*);
extern unitptr nst_begin_rgb_draw(nst_winfo_t*);
extern unitptr nst_begin_rgb_3d_draw(nst_winfo_t*);
extern void nst_end_draw(nst_winfo_t*);

extern int nst_define_subwindow(void);
extern int nst_reopen_subwindow(int,char*,unitptr);
extern int nst_reopen_floating_subwindow(int,int,char*,unitptr);
extern void nst_move_subwindow(int,int,int,int);
extern void nst_resize_subwindow(int,int,int,int,int,int);
extern void nst_remove_windows_of(unitptr);
extern void nst_remove_vacant_subwindows_of(unitptr);

extern void nst_set_window_notify_functions(void (*pNotifyFunction1)(int),
                                            void (*pNotifyFunction2)(int));

extern void nst_resize_notify(int iId);
extern void nst_resize_window(int, int, int, int);

extern void nst_defer_redraws(void);
extern void nst_do_deferred_redraws(void);

extern int nst_window_is_open(void);
extern void nst_abandon_windows(void);

extern void nst_ctrl_attached_units(int,unitptr);
extern void nst_ctrl_all_attached_units(int,unitptr);
extern void nst_ctrl_all_embedded_clients(int,unitptr);
extern void nst_ctrl_clients_of_window(int,int);

extern int nst_get_symbol(void);
extern int nst_get_color(void);
extern int nst_get_size(void);
extern int nst_get_hotspot(void);

extern int gl_to_nst_color(int);
extern int nst_to_gl_color(int);

extern void nst_mask_windows(void);
extern void nst_unmask_windows(void);
extern void nst_map_windows_of(unitptr);
extern void nst_unmap_windows_of(unitptr);
extern void nst_map_all_windows(unitptr);
extern void nst_unmap_all_windows(unitptr);
extern void nst_cyclic_swap_wdw_queue(unitptr,unitptr,unitptr);

extern void nst_dat_to_pix2f(float,float,float*,float*);
extern void nst_dat_to_pix2i(float,float,int*,int*);
extern void nst_pix_to_dat2f(int,int,float*,float*);
extern void nst_pix_to_dat2i(int,int,int*,int*);

extern nst_winfo_t *nst_make_winfo(char*,unitptr);
extern nst_winfo_t *nst_init_winfo(char*,char*,nst_winfo_t*);
extern int nst_free_winfo0(nst_winfo_t*);
extern int nst_free_winfo(nst_winfo_t*);

extern void nst_main_window_client(nst_winfo_t*);
extern void nst_finish_clear(nst_winfo_t*);
extern int nst_is_window_host(unitptr);
extern int nst_is_subunit(unitptr, unitptr);
extern unitptr nst_shared_host(unitptr, unitptr); 

extern  void gprintf(int*,int*,char,int);
extern  void nst_charstr(char*);

extern  void set_RGB_color(int);
extern  void nst_register_window(int,int,int,int,char*,void*);
extern  void nst_define_window(int,int,int,int,int,char*,void*);
extern  void nst_register_subwindow(int,int,int,int,int,int,int,char*,void*);
extern  int nst_is_subwindow(int);
extern  nst_winfo_t *nst_host_for_subwindow(char*,unitptr);

extern  int nst_find_window_from_str(char*,unitptr);
extern  int nst_find_window_or_subwindow_from_u(unitptr);
extern  int nst_find_window_from_u(unitptr u);
extern  int nst_find_window_from_u_or_subunit(unitptr u);
extern  int nst_find_associated_window(char*,unitptr);

extern  unitptr nst_find_u_from_window(int iWdw);
extern  unitptr nst_find_u_or_embeddor_from_window(int iWdw);
extern  unitptr nst_main_window_host(nst_winfo_t*);

extern  unitptr successor_of_u(unitptr);
extern  unitptr predecessor_of_u(unitptr);

extern  void clr_window(int,int);
extern  void rm_wdw_qentry(int);
extern  void nst_swap_windows_of(int,int);
extern  void swap_wdw_qentries(unitptr,unitptr);
extern  void nst_draw_tile(int, int, int, int, int, int, int);
extern  void nst_draw_frame(int, int, int, int, int , int);
extern  void nst_draw_slider(int, int,int,int, float);
extern  void nst_highlight_slider(int, int,int,int, float);
extern  void nst_draw_vslider(int, int,int,int, float);
extern  void nst_highlight_vslider(int, int,int,int, float);
extern  void nst_draw_scrollbar(int xll, int yll,int xur,int yur, 
				   float ylow, float yhigh  /* relative to yll */);
extern  void nst_highlight_scrollbar(int xll, int yll,int xur,int yur, 
				   float ylow, float yhigh  /* relative to yll */);
extern  void nst_draw_scrollbar_arrows(int xll, int yll,int xur,int yur);
extern  void nst_highlight_scrollbar_arrows(int xll, int yll,int xur,int yur,int which);
extern  int  nst_draw_button(int,int,int,int,int,char *);
extern  int  nst_highlight_button(int,int,int,int,int,char *);
extern  int  nst_draw_shaded_button(int,int,int,int,int,char *);
extern  int nst_draw_zoom_widget(int,int,int,int,int,int,int,int,char*);
extern  void nst_draw_value(int,int, int, int, float, char *);
extern  void nst_draw_text(int,int,int, void*, int,int);
extern  void nst_draw_corner(int,int,int,int,int,int,int,int);
extern  void nst_draw_hrule(int iX, int iY, int iPixlen);
extern  int  nst_selects_corner(int,int,int,int,int,int,int);
extern  void nst_highlight_text(int,int,int, void*, int,int);
extern  void nst_close_windows_of(unitptr);
extern  void nst_gflush(int iPen);
extern int nst_unparent_subwindows_of(unitptr uEmbeddor);

extern void nst_draw_str(int,int,int,int,char*,int);
extern int nst_get_geometry(int,int*,int*,int*,int*);
extern int nst_label_maxwidth(char*);

extern void nst_set_type_of_registered_window(int);

extern void print_nst_wdwqueue(void);
extern int  nst_track_pointer(int iNumWindows, char **ppcWindow,
                              float *pfX1, float *pfY1,
                              float *pfX2, float *pfY2,
                              int *piButtons,
                              int iFrameColor,
                              void (*pFrame)(float,float,float,float),
                              unitptr u);
#endif
