/***********************************************************************

 Version 4.0 of include file nst_file.h 

 Copyright by Helge Ritter 1995 

***********************************************************************/
#ifndef __NST_FILE_H
#define __NST_FILE_H

#ifndef NST_F_FLUSH
#define NST_F_FLUSH -39   /* control mode to flush opened files */
#endif

unittype *read_file(int n, char* fileid, char* fmt1, char* fmt2, int nr,
    unittype *dest);

/*  assigns values to x[] that are read from file fp according to
    the format specified in fmt.
    Return value: number of assigned items, -1 if EOF was reached.
    Format specification:

       fmt = "field_1 [field_2 ...]"

    The fields field_i are delimited by one or more blanks.
    Each field_i is either a text token or a control field.

    A text token is simply a consecutive string of characters
    (delimited by blanks) that must not begin with a single %
    (such as %forbidden). A text token advances the file pointer
    immediately behind the next occurrence of the same text token in
    the file, or to EOF, if there is no further occurrence.

    A control field is any field beginning with a single %.
    (such as %34.5) and delimited by blanks. The following kinds
    of control fields are available (d1 and d2 denote nonnegative
    integers without sign):
       %d1-d2  :  assign the next d2-d1+1 float values in the
		   file to x[d1]..x[d2].
       %d1     :  assign next float value in the file to x[d1].
       %sd1    :  skip next d1 tokens (text or numbers) in the file.
       %n      :  skip rest of current line.
       %d1.d2  :  use format %d1.d2f for remaining writes
		   (not used for reading)
    If a token that is expected to represent a float value is
    instead a text token, a value of 0 is read.

    Examples:
       "%s5 %0-10  %n %11-20"   skip first 5 tokens, assign next
		eleven tokens to x[0..10], skip to next line and
		assign first 10 tokens to x[11..20].

       "eps: %3  sigma1,sigma2 %1-2 %n %s4 %0"
		Assign to x[3] first float item after token "eps",
		to x[1],x[2] the two numbers following the next
		occurrence of "sigma1,sigma2", and assign the 5th
		token on the next line to x[0].
 */

unittype *write_file(int n, char* fileid, char* fmt1, char* fmt2, 
    unittype *dest);

/* Creates a unit for carrying out file-I/O with file fileid.
   Opens file fileid, and writes data according to fmt1.
   
   Each subsequent exec call the performs a write of the current
   input values selected and formatted according to fmt2.

*/


unitptr read_pxmfile(int dx, int dy, char* fileid, unitptr dest);

/* creates a unit with a single packed output field of dx*dy
pins. The exec-routine of this unit opens
and reads a pbm, pgm or ppm-file with fileid "fileid". The image region
defined by the file is intersected with a rectangle of
width dx and height dy (lower left corners of both regions
are aligned) and the resulting intersection defines
the values of the dx*dy pins of the packed output field of
this unit. Pixels of the dx*dy-image region that are not
specified by the file are set to a value of zero.
If the file is a ppm-file, the returned pixel values are
the average of the three color channels. Pixel intensities
are always normalized to the range 0 (black) to 255 (white).
Between successive
reads, fileid may be changed to switch between different
files.
*/


unitptr write_pgmfile(int dx, int dy, char* fileid, unitptr dest);

/* creates a unit with a single packed input field of dx*dy
pins. The exec-routine of this unit creates
and writes a pgm-file with fileid "fileid".
for an image region of width dx and height dy.
Between writes, fileid may be changed to switch between different
files.
*/


unitptr write_ppmfile(int dx, int dy, char* fileid, unitptr dest);

/* analogous to write_pgmfile, but with 3 packed input fields
   for writing RGB-image. */

unitptr multi_ppm(int,int,char*,int,int,unitptr);


extern unitptr read_feature_file(int,char*,unitptr);

extern unitptr read_image(int,int,char*,unitptr);
extern unitptr read_data(char*,char*,unitptr);
extern unitptr write_data(
   char *pcFileName,  /* file name for output file */
   char *pcFmt,     /* format string */
   char *pcHeader,  /* header to be prepended */
   unitptr uHost    /* host unit */
);

extern unitptr write_image_dyn(
   char *pcFileName,  /* file name for output file */
   char *pcFmt,     /* format string */
   char *pcHeader,  /* header to be prepended */
   unitptr uHost    /* host unit */
);

extern unitptr read_image_dyn(
   char *pcFileName,  /* file name for output file */
   char *pcFmt,     /* format string */
   char *pcHeader,  /* prepended header */
   unitptr uHost    /* host unit */
);


extern unitptr read_pxmfile8(int,int,char*,unitptr);
extern unitptr read_image8(int,int,char*,unitptr);


#endif
