/******************************************************************************
 *
 * File nst_meta.h: 
 * headers for 'Meta' units that modify other units or wires.
 * currently, only a remake_unit for modifying operands is provided.
 *
 * $Source: /vol/ni/src/cvsroot/nst5/nstsrc/nst_meta.h,v $
 * $Log: nst_meta.h,v $
 * Revision 1.8  2003/12/04 12:47:05  rhaschke
 * replaced var_list by array of union of several types
 *
 * Revision 1.7  2000/02/01 15:30:22  helge
 * added %PRESERVE option to remake_unit, allowing to exclude a subset of
 * operand subunits from the remake operation.
 *
 * Revision 1.6  1997/12/20 23:52:16  helge
 * - remake_unit: remade unit no longer restricted to immediate successor:
 *                %USEUNIT directive (see Manpage) allows
 *                         a) to remake any named operand,
 *                         b) to remake operand of host unit,
 *                         c) to remake named unit within host unit
 *                [can make containers inherit remake operator property]
 *
 * Revision 1.5  1997/09/28 18:19:53  helge
 * added some missing defs
 *
 * Revision 1.4  1997/04/17 10:20:52  jan
 * Corrected function prototype.
 *
 * Revision 1.3  1997/04/02 14:09:23  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.2  1997/03/13  10:55:20  jan
 * Dependency cleanup.
 *
 * Revision 1.4  1997/01/20 13:48:15  helge
 * various minor changes.
 *
 * Revision 1.3  1997/01/14  14:25:50  helge
 * numerous small changes to take care of (almost) all warnings under
 * -Wall -O2.
 *
 * Revision 1.2  1996/12/09 19:46:48  helge
 * new package nst_meta for dynamically remaking units.
 *
 * Revision 1.1  1996/11/16 19:16:12  helge
 * 3 groups of extensions:
 * a) new module nst_meta.c with remake_unit; accompanying changes
 *    in nst1-3.c (now contain some former neo routines)
 * b) minor changes to david.c (safer error handling)
 * c) improved scanf_unit: recognizes new non-standard format specifiers:
 *    %dv %dV     read vector (packed for 'V') of dim d
 *    %w:hm %w:hM read mono image rectangle (byte/float for m/M) of width w,
 *                height h from pnm file with name given by current token
 *    %w:hc %w:hC dito, but for color image
 *
 *
 *****************************************************************************/

#ifndef __NST_META_H
#define __NST_META_H

#include "nst_var_list.h"

extern unitptr nst_find_referenced_unit(char*,unitptr);

#define P_exclude     (0x1UL<<17)

typedef struct var_remake_unit_{ /* local var of unit neo_remake_unit: */
  int iNum, iNumericItems, iTextItems, iSlots, iNumPreserved;
  char  *pcName, *pcNewName, **ppcExclude, *pcPinType, **ppcTextItem;
  float *pfNumericItem, *pfTemp;
  int   *piSlot;
  unitptr uOld;
  unitptr (*pRemakeFunction)(unitptr);  /* for NST-version */
  nst_var_list vArg;
} var_remake_unit_t;

extern unitptr nst_find_remake_operand(char*,unitptr);
extern unitptr remake_unit_internal(char*,unitptr(*)(unitptr),int,nst_var_list,unitptr);
extern unitptr remake_unit(char*,unitptr(*)(unitptr),nst_var_list,unitptr);
extern nst_var_list nst_wrap_args_of(char*,...);
extern nst_var_list nst_remake_args_of(unitptr);
#endif
