/************************************************************************
 * This header file describes utilities for rpc units
 * former location nst_rpc.x -> .h
 ************************************************************************
 * $Source: /vol/ni/src/cvsroot/nst5/nstsrc/nst_rpc_util.h,v $
 * $Log: nst_rpc_util.h,v $
 * Revision 1.3  1997/04/02 14:10:34  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.2  1997/03/13  10:56:02  jan
 * Dependency cleanup.
 *
 * Revision 1.6  1997/01/20 13:49:20  helge
 * various minor changes.
 *
 * Revision 1.5  1995/07/31  09:23:51  helge
 * some minor changes.
 *
 * Revision 1.3  1995/07/16 20:55:36  helge
 * enhanced rpc_unit to make remote units from neo's *.NST-files
 * extended nst_rpd.c, nst_rpd_add accordingly [changes disabled by
 * undefing NEO in nst_rpd_add.c]
 * provided 8-bit versions of some nst-units [nst_math8 etc.]
 *
 * Revision 1.2  1994/11/10  11:21:15  walter
 * NOMAIL, support multiple clients rpc_clnt_init_byNum
 *
 * Revision 1.1  1994/02/18  18:19:08  walter
 * created nst_rpc_util.[ch] for rpc_utilities like special print and
 * debug routines. cleaned makefile, added $C to CFLAGS and $L to LDFLAGS
 * for easy compiler option access.
 *
 ************************************************************************/
#ifndef __NST_RPC_UTIL_H
#define __NST_RPC_UTIL_H

#define  RNSTCTRL_MODE_TRON		+1
#define  RNSTCTRL_MODE_TROFF		-1
#define  RNSTCTRL_MODE_LIST_AC_UNITS	2
#define  RNSTCTRL_MODE_LIST_SRV_TYPS	20
#define  RNSTCTRL_MODE_DEL_OBSOLETE_UNITS_SIL	3
#define  RNSTCTRL_MODE_DEL_OBSOLETE_UNITS_VER	30
#define  RNSTCTRL_MODE_KILL_SERVER	-9999
#define  RPC_EXIT_ON_ERROR	0x10	/* bit mask for iDbx */

#define SAY(x) printf("%s:%d: %s\n", __FILE__,__LINE__,(x));

typedef struct trsp_data_{
   unsigned char *pcData;   /* array where byte els are written   */
   NstInt_t  *piData;   /* array where int els are written   */
   float *pfData;  /* array where float els are written   */
   char  *ptData;  /* array where char strings els are written
                      [separated by null bytes]   */
   int iNumc,iNumi,iNumf,iNumt;  /* currently used nr of array els */
   int iMaxc,iMaxi,iMaxf,iMaxt;  /* currently avail nr of array els */
   int *piPins;   /* entries in this array identify to which pins
                     the contents of the arrays .. belongs.
                     first el is total nr of entries that follow [not
                     including the el 0 itself]
                     remaining entries:
                     if neg value i<0: change current field to new value
                                       field = (current field) + abs(i)
                           [initially, current field = 0]
                     otherwise:
                           array contents belongs to pin i of current field.
                     Type info must be taken from description of the
                     identified pin.      
                  */
} trsp_data;

typedef struct _srvRpcUnit { /* Unit struct on server side */
	rpcCreateReqStruc q;		/* complete request stuct */ 
	int id;				    /* Unit's remote id */
    unitlistel *opqueue;      /* a pseudo-queue. If u2 is an operator, this
                               queue contains references to operands. */
	unittype * u;			/* nst_unit  */
	trsp_data *pData;       /* data interface for unwrap */
	char label[COMMENTSTRINGMAX];	/* label */
	struct _srvRpcUnit *next,*prev;	/* neighbors srvRpcUnit in list */
	CLIENT * clntPpingHndl;		/* pping server handle to client */
    } srvRpcUnit; 

#ifndef NST_RCC				/* ANSI compiler */
    extern CLIENT * rpc_clnt_init_byNum(char * hostname, long lRpcPort);
    extern CLIENT * rpc_clnt_init(char * hostname);
    extern void free_trsp_data(trsp_data*);
    extern int sumPins(int *f);
    extern unitptr merger2( int *fdesc , unitptr dest);
    extern unitptr splitter2( int *fdesc , unitptr dest);
    extern unitptr rpc_unit(CLIENT *cl,int type, int ac, char **av, unitptr dest);
    extern void print_rpcCreateReqStruc(rpcCreateReqStruc* r, char* com);
    extern void print_rpcCreateResStruc(rpcCreateResStruc* r, char* com);
    extern void print_rpcTranspStruc(rpcTranspStruc* r, char* com);
    extern int print_puma(unitptr u);
    extern int setRpcDbx(int i);	/* in 2 files nst_rp[cd].c */
    extern int getRpcDbx(void);
#else 
    extern CLIENT * rpc_clnt_init();		/* KR - compiler friendly */
    extern CLIENT * rpc_clnt_init_byNum();
    extern void free_trsp_data();
    extern int sumPins();
    extern unitptr merger2();
    extern unitptr splitter2();
    extern unitptr rpc_unit();
    extern void print_rpcCreateReqStruc();
    extern void print_rpcCreateResStruc();
    extern void print_rpcTranspStruc();
    extern int print_puma();
    extern int setRpcDbx();		/* in 2 files nst_rp[cd].c */
    extern int getRpcDbx();
#endif
extern char acReturnComment[];

#endif
